﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using Microsoft.AspNetCore.Mvc;
using Microsoft.AspNetCore.Mvc.Rendering;
using ELTE.TodoList.Web.Services;
using ELTE.TodoList.Web.Models;
using Microsoft.AspNetCore.Http;
using System.IO;

namespace ELTE.TodoList.Web.Controllers
{
    public class ItemsController : Controller
    {
        private readonly ITodoListService _service;

        public ItemsController(ITodoListService service)
        {
            _service = service;
        }

        public IActionResult DisplayImage(int id)
        {
            var item = _service.GetItem(id);
            if (item == null)
                return null;

            return File(item.Image, "image/png");
        }

        // GET
        public IActionResult Create()
        {
            ViewBag.Lists = new SelectList(_service.GetLists(), "Id", "Name", TempData["ListId"]);
            return View();
        }

        [HttpPost]
        [ValidateAntiForgeryToken]
        public IActionResult Create(ItemViewModel vm, IFormFile image)
        {
            var item = (Item)vm;

            if (image != null && image.Length > 0)
            {
                using (var stream = new MemoryStream())
                {
                    image.CopyTo(stream);
                    item.Image = stream.ToArray();
                }
            }

            if (ModelState.IsValid)
            {
                _service.CreateItem(item);
                return RedirectToAction("Details", "Lists", new { id = item.ListId });
            }

            ViewBag.Lists = new SelectList(_service.GetLists(), "Id", "Name", item.ListId);
            return View(item);
        }

        // GET
        public IActionResult Edit(int? id)
        {
            if (id == null)
            {
                return NotFound();
            }

            var item = _service.GetItem((int)id);
            if (item == null)
            {
                return NotFound();
            }

            ViewBag.Lists = new SelectList(_service.GetLists(), "Id", "Name", item.ListId);
            return View(item);
        }

        [HttpPost]
        [ValidateAntiForgeryToken]
        public IActionResult Edit(int id, ItemViewModel vm, IFormFile image)
        {
            if (id != vm.Id)
            {
                return NotFound();
            }

            var item = (Item)vm;

            if (image != null && image.Length > 0)
            {
                using (var stream = new MemoryStream())
                {
                    image.CopyTo(stream);
                    item.Image = stream.ToArray();
                }
            }

            if (ModelState.IsValid)
            {
                bool result;
                result = _service.UpdateItem(item);

                if (result)
                {
                    return RedirectToAction("Details", "Lists", new { id = item.ListId });
                }
                else
                {
                    ModelState.AddModelError("", "Hiba történt a mentés során!");
                }
            }

            ViewBag.Lists = new SelectList(_service.GetLists(), "Id", "Name", item.ListId);
            return View(item);
        }

        // GET
        public IActionResult Delete(int? id)
        {
            if (id == null)
            {
                return NotFound();
            }

            var item = _service.GetItem((int)id);
            if (item == null)
            {
                return NotFound();
            }

            return View(item);
        }

        [HttpPost, ActionName("Delete")]
        [ValidateAntiForgeryToken]
        public IActionResult DeleteConfirmed(int id)
        {
            var item = _service.GetItem(id);
            if (item != null)
            {
                _service.DeleteItem(id);
                return RedirectToAction("Details", "Lists", new { id = item.ListId });
            }
            else
            {
                return NotFound();
            }
        }
    }
}